/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.usm.session.dataobject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.UUID;
import com.openexchange.usm.api.session.DataObject;

public class DataObjectSet implements Collection<DataObject> {

	private final static String MISSING_ID_PREFIX = "DATAOBJECT_WITHOUT_ID_";

	private final Map<String, DataObject> _idMap;
	private final Map<UUID, DataObject> _uuidMap;
	private int _noIDCounter = 0;

	public DataObjectSet() {
		_idMap = new HashMap<String, DataObject>();
		_uuidMap = new HashMap<UUID, DataObject>();
	}

	public DataObjectSet(int size) {
		_idMap = new HashMap<String, DataObject>(size);
		_uuidMap = new HashMap<UUID, DataObject>(size);
	}

	public DataObjectSet(DataObject[] elements) {
		this(elements == null ? 100 : elements.length);
		if (elements != null)
			addAll(elements);
	}

	public DataObjectSet(Collection<? extends DataObject> elements) {
		this(elements == null ? 100 : elements.size());
		if (elements != null)
			addAll(elements);
	}

	@Override
    public boolean add(DataObject e) {
		String id = e.getID();
		if (id == null)
			id = generateNoIDKey(_noIDCounter++);
		_idMap.put(id, e);
		UUID uuid = e.getUUID();
		if (uuid != null)
			_uuidMap.put(uuid, e);
		return true;
	}

	@Override
    public boolean addAll(Collection<? extends DataObject> c) {
		if (c == null || c.size() == 0)
			return false;
		for (DataObject o : c)
			add(o);
		return true;
	}

	public boolean addAll(DataObject[] c) {
		if (c == null || c.length == 0)
			return false;
		for (DataObject o : c)
			add(o);
		return true;
	}

	@Override
    public void clear() {
		_idMap.clear();
		_uuidMap.clear();
	}

	public boolean contains(String id) {
		return _idMap.containsKey(id);
	}

	public boolean contains(UUID uuid) {
		return _uuidMap.containsKey(uuid);
	}

	public DataObject get(String id) {
		return _idMap.get(id);
	}

	public DataObject get(UUID uuid) {
		return _uuidMap.get(uuid);
	}

	@Override
    public boolean isEmpty() {
		return _idMap.isEmpty();
	}

	@Override
    public Iterator<DataObject> iterator() {
		return _idMap.values().iterator();
	}

	public DataObject remove(String id) {
		DataObject o = _idMap.remove(id);
		if (o != null) {
			UUID uuid = o.getUUID();
			if (uuid != null)
				_uuidMap.remove(uuid);
		}
		return o;
	}

	public DataObject remove(UUID uuid) {
		DataObject o = _uuidMap.remove(uuid);
		if (o != null) {
			String id = o.getID();
			if (id != null)
				_idMap.remove(id);
		}
		return o;
	}

	@Override
    public int size() {
		return _idMap.size();
	}

	@Override
    public DataObject[] toArray() {
		Collection<DataObject> values = _idMap.values();
		return values.toArray(new DataObject[values.size()]);
	}

	public List<DataObject> toList() {
		List<DataObject> list = new ArrayList<DataObject>(_idMap.size());
		for (DataObject o : this)
			list.add(o);
		return list;
	}

	/**
	 * This method only checks if a DataObject with the same id is stored in this DataObjectSet.
	 * It does not check for equality of the DataObjects themselves.
	 * 
	 */
	@Override
    public boolean contains(Object o) {
		if (!(o instanceof DataObject))
			return false;
		DataObject ob = get(((DataObject) o).getID());
		if (ob != null)
			return true;
		if (_noIDCounter > 0) {
			for (int i = 0; i < _noIDCounter; i++) {
				ob = get(generateNoIDKey(i));
				if (o == ob)
					return true;
			}
		}
		return false;
	}

	@Override
    public boolean containsAll(Collection<?> c) {
		for (Object e : c) {
			if (!contains(e))
				return false;
		}
		return true;
	}

	@Override
    public boolean remove(Object o) {
		if (!(o instanceof DataObject))
			return false;
		DataObject ob = remove(((DataObject) o).getID());
		DataObject ob2 = null;

        if (ob == null) {
            for (Entry<String, DataObject> entry : _idMap.entrySet()) {
                if (entry.getValue() == o) {
                    ob = _idMap.remove(entry.getKey());
                    break;
                }
            }
            for (Entry<UUID, DataObject> entry : _uuidMap.entrySet()) {
                if (entry.getValue() == o) {
                    ob2 = _uuidMap.remove(entry.getKey());
                    break;
                }
            }
        } else {
            ob2 = ob;
        }


		//		if (ob == null && _noIDCounter > 0) {
		//			for (int i = 0; i < _noIDCounter; i++) {
		//				String key = generateNoIDKey(i);
		//				DataObject ob2 = get(key);
		//				if (ob2 == o) {
		//					_idMap.remove(key);
		//					break;
		//				}
		//			}
		//		}

		return ob != null && ob2 != null;
	}

	@Override
    public boolean removeAll(Collection<?> c) {
		boolean result = false;
		for (Object o : c) {
			result |= remove(o);
		}
		return result;
	}

	@Override
    public boolean retainAll(Collection<?> c) {
		throw new UnsupportedOperationException();
	}

	@Override
    public <T> T[] toArray(T[] a) {
		throw new UnsupportedOperationException();
	}

	@Override
	public String toString() {
		return _idMap.toString();
	}

	private String generateNoIDKey(int n) {
		return MISSING_ID_PREFIX + n;
	}

	/**
	 * Checks whether this DataObjectSet contains exactly the given DataObjects, i.e. same number, same IDs of
	 * DataObjects, every ID must be present and the local DataObject with the given ID must be equal to the given
	 * DataObject (i.e. same ChangeState,all fields equal, timestamps *do not* need to be equal) and the contained
	 * ProtocolInformations and UUID must also be equal.
	 * 
	 * @param objects
	 * @return
	 */
	public boolean isEqualTo(DataObject[] objects) {
        if(objects == null)
            return isEmpty();
        if (size() != objects.length)
            return false;
        for (DataObject o : objects) {
            if (o == null)
                return false;
            DataObject local = get(o.getID());
            if (!o.equalsWithLastModified(local) || !DataObjectUtil.isEqual(o.getProtocolInformation(), local.getProtocolInformation())
                    || !DataObjectUtil.isEqual(o.getUUID(), local.getUUID()))
                return false;
        }
        return true;
    }

    public Set<String> getIDs() {
        return _idMap.keySet();
    }
}
